import { Component, NgZone } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Device } from '@ionic-native/device/ngx';
import { LocalNotifications } from '@ionic-native/local-notifications/ngx';
import { Network } from '@ionic-native/network/ngx';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { Platform } from '@ionic/angular';
import { FCM } from 'cordova-plugin-fcm-with-dependecy-updated/ionic/ngx';
import { AuthService } from 'ionic-appauth';

import { IdpInformationService, ManagePushService, StorageService, UtilityService } from './services';
import { AuthenticationService } from './services/authentication/authentication.service';
import { PAGES } from './utility';

@Component({
    selector: 'app-root',
    templateUrl: 'app.component.html',
})
export class AppComponent {
    constructor(
        private authService: AuthenticationService,
        private platform: Platform,
        private splashScreen: SplashScreen,
        private statusBar: StatusBar,
        private localNotifications: LocalNotifications,
        private auth: AuthService,
        private storageService: StorageService,
        private managePushService: ManagePushService,
        private fcm: FCM,
        private device: Device,
        private network: Network,
        private router: Router,
        private activatedRoute: ActivatedRoute,
        private utilityService: UtilityService,
        private idpInformationService: IdpInformationService,
        private ngZone: NgZone) {
        this.subscribeToConnection();
        this.initializeApp();
    }

    async ngOnInit() {
        this.authService.isLoggedIn().then((isLoggedin) => {
            if (isLoggedin) {
                this.authService.loggedIN$.next(true);
            } else {
                this.authService.loggedIN$.next(false);
            }
        });
    }

    initializeApp() {
        this.platform.ready().then(async () => {
            this.splashScreen.hide();
            await this.auth.init();
            this.checkStatusBar();
            this.manageTokens();
            if (this.device.platform == 'iOS') {
                this.initializeFCMiOS();
                this.clickOnNotification();
            } else if (this.device.platform == 'Android') {
                this.initializeFCMAndroid();
                this.clickOnNotification();
            }
            setTimeout(() => { this.getInitialPushPayload(); });
        });
    }

    checkStatusBar() {
        if (this.platform.is('ios')) {
            this.statusBar.styleDefault();
        } else if (this.platform.is('android')) {
            this.statusBar.styleLightContent();
        }
    }

    manageTokens() {
        this.authService.isLoggedIn().then(async (isLoggedin) => {
            if (isLoggedin) {
                //this.idpInformationService.refreshToken().subscribe(() => {});
                this.utilityService.sendTokenFirebase();
            } else {
                //se non loggato genera silentemente il token che permettedi chiamare l'api manager delle news
                await this.idpInformationService.generateSilentToken().toPromise();
            }
        });
    }

    initializeFCMiOS() {
        this.fcm.onNotification().subscribe((data) => {
            if (data.wasTapped) {
                this.manageBackgroundNotifications(data.codice, data.valore);
            } else {
                this.manageForegroudNotifications(data?.aps?.alert?.title, data?.aps?.alert?.body, data.codice, data.valore);
            }
        });
        this.fcm.requestPushPermission();
    }

    initializeFCMAndroid() {
        this.fcm.onNotification().subscribe((data) => {
            if (data.wasTapped) {
                this.manageBackgroundNotifications(data.codice, data.valore);
            } else {
                this.manageForegroudNotifications(data.title, data.body, data.codice, data.valore);
            }
        });
    }

    clickOnNotification() {
        this.localNotifications.on('click').subscribe(
            (notification) => {
                if (notification.data.codice === 'MACS_PRE') {
                    this.managePushService.goToDettaglioPrenotazione(notification.data.valore);
                }
                if (notification.data.codice === 'MACS_COM') {
                    this.managePushService.getComunicazione(notification.data.valore);
                }
                if (notification.data.codice === 'MACS_REM') {
                    this.ngZone.run(() => {
                        this.router.navigate([PAGES.landing_page], {
                            relativeTo: this.activatedRoute,
                        });
                    });
                }
                if (notification.data.codice === 'MACS_NEWS') {
                    this.managePushService.goToDetailNews(notification.data.valore);
                }
                if (notification.data.codice === 'MACS_GEST_PRA') {
                    this.managePushService.goToDetailProcedure(notification.data.valore);
                }
            },
            (error) => {
                console.log('localNotifications.on()', error);
            }
        );
    }

    manageBackgroundNotifications(codice: string, valore: string) {
        if (codice === 'MACS_PRE') {
            this.authService.isLoggedIn().then((isLoggedin) => {
                if (isLoggedin) {
                    this.managePushService.goToDettaglioPrenotazione(Number(valore));
                } else {
                    this.ngZone.run(() => {
                        this.router.navigate([PAGES.landing_page], {
                            relativeTo: this.activatedRoute,
                        });
                    });
                }
            });
        }
        if (codice === 'MACS_COM') {
            this.managePushService.getComunicazione(Number(valore));
        }
        if (codice === 'MACS_REM') {
            this.ngZone.run(() => {
                this.router.navigate([PAGES.landing_page], {
                    relativeTo: this.activatedRoute,
                });
            });
        }
        if (codice === 'MACS_NEWS') {
            this.managePushService.goToDetailNews(valore);
        }
        if (codice === 'MACS_GEST_PRA') {
            this.managePushService.goToDetailProcedure(valore);
        }
    }

    manageForegroudNotifications(title: string, body: string, codice: string, valore: string) {
        if (codice === 'MACS_PRE') {
            this.localNotifications.schedule({
                id: Math.random() * 99999,
                title: title,
                text: body,
                lockscreen: true,
                foreground: true,
                data: { codice: codice, valore: valore },
            });
        }
        if (codice === 'MACS_COM') {
            this.localNotifications.schedule({
                id: Math.random() * 99999,
                title: title,
                text: body,
                lockscreen: true,
                foreground: true,
                data: { codice: codice, valore: valore },
            });
        }
        if (codice === 'MACS_REM') {
            this.authService.isLoggedIn().then((isLoggedin) => {
                if (isLoggedin) {
                    this.localNotifications.schedule({
                        id: Math.random() * 99999,
                        title: title,
                        text: body,
                        lockscreen: true,
                        foreground: true,
                        data: { codice: codice },
                    });
                }
            });
        }
        if (codice === 'MACS_NEWS') {
            this.localNotifications.schedule({
                id: Math.random() * 99999,
                title: title,
                text: body,
                lockscreen: true,
                foreground: true,
                data: { codice: codice, valore: valore },
            });
        }
        if (codice === 'MACS_GEST_PRA') {
            this.localNotifications.schedule({
                id: Math.random() * 99999,
                title: title,
                text: body,
                lockscreen: true,
                foreground: true,
                data: { codice: codice, valore: valore },
            });
        }
    }

    subscribeToConnection() {
        this.storageService.storeConnectionStatus(true);
        this.network.onConnect().subscribe(() => {
            console.log('network connected!');
            this.storageService.storeConnectionStatus(true);
        });
        this.network.onDisconnect().subscribe(() => {
            console.log('network was disconnected :-(');
            this.storageService.storeConnectionStatus(false);
        });
    }

    storeData() {
        this.storageService.storeUserInfo({
            identifier: 'SLTMRA80A01A944H',
            // identifier: "a.lodato",
            firstName: 'Saltelli',
            lastName: 'Mario',
            sub: 'SLTMRA80A01A944H',
            email: 'daniele.sampaoli2@maggioli.it',
        });
    }

    getInitialPushPayload(){
        this.fcm.getInitialPushPayload().then((data) => {
            if (data) {
                if (data.wasTapped) {
                    this.manageBackgroundNotifications(data.codice, data.valore);
                } else {
                    this.manageForegroudNotifications(data.title, data.body, data.codice, data.valore);
                }
            }
        });
    }

}
