import { Component, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { IonContent, Platform } from '@ionic/angular';
import * as SockJS from 'sockjs-client';
import { TransformPipe } from 'src/app/pipe';
import * as Stomp from 'stompjs';

import { MessageChat, ResponseChatBot } from '../../../models';
import { ASCII_CHAR_REGEX, ASCII_CHARACTER, CHAT_API, CHATBOT, IMAGE, MESSAGES, OPEN_WITH_EXTERNAL_BROWSER } from '../../../utility';

@Component({
    selector: 'app-chat',
    templateUrl: './chat.page.html',
    styleUrls: ['./chat.page.scss'],
})
export class ChatPage implements OnInit {
    @ViewChild(IonContent) content: IonContent;
    titleHeader: string = 'Alessia';
    subTitleHeader: string = 'Assistente intelligente';
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    ios: boolean;
    android: boolean;
    messageForm: FormGroup;
    listaMessages: Array<MessageChat>;
    textMessage: string = MESSAGES.chatUnavailable;
    chatUnavailable: boolean = false;
    socketNotReady: boolean = true;
    showSkeletonBot: boolean = true;
    //CHAT
    stompClient: any;

    constructor(private platform: Platform, private formBuilder: FormBuilder, private inAppBrowser: InAppBrowser, private transformPipe: TransformPipe) {
        this.checkPlatform();
        this.creaFrom();
        this.setUtility();
        //CHAT
        this.configSock();
    }

    ngOnInit(): void {
        this.callServicesChat();
        this.startTimer();
    }

    setUtility() {
        this.listaMessages = [];
    }

    configSock() {
        const socket = new SockJS(CHAT_API);
        console.log('CHAT_API:', CHAT_API);
        console.log('socket:', socket);
        this.stompClient = Stomp.over(socket);
    }

    creaFrom() {
        this.messageForm = this.formBuilder.group({
            message_sent: new FormControl(''),
        });
    }

    checkPlatform() {
        this.ios = this.platform.is('ios');
        this.android = this.platform.is('android');
    }

    get message_sent() {
        return this.messageForm.get('message_sent').value;
    }

    set message_sent(text: string) {
        this.messageForm.get('message_sent').setValue(text);
    }

    public sendMessage(): void {
        this.addToListaMessages(null, true);
        this.handleMessage(this.message_sent);
        this.message_sent = '';
    }

    callServicesChat() {
        this.stompClient.connect(
            {},
            () => {
                console.log('1');
                this.chatUnavailable = false;
                this.showSkeletonBot = false;
                this.stompClient.subscribe('/user/queue/message', (msg) => {
                    this.addToListaMessages(JSON.parse(msg.body) as ResponseChatBot);
                });
                this.stompClient.subscribe('/user/queue/configuration', (msg) => {
                    msg = JSON.parse(msg.body);
                    if (msg.result) {
                        this.socketNotReady = false;
                        this.stompClient.send('/message', {}, this.handleMessage('/get_started'));
                    }
                });
                this.stompClient.subscribe('/user/queue/error', (msg) => {
                    msg = JSON.parse(msg.body);
                    this.chatUnavailable = true;
                });
                this.stompClient.send('/configuration', {}, JSON.stringify({ type: 'local' }));
            },
            () => {
                console.log('2 a');
                this.listaMessages = [];
                this.chatUnavailable = true;
                this.showSkeletonBot = false;
            }
        );
    }

    handleMessage(textMsg: string) {
        this.stompClient.send('/message', {}, this.getMessage(textMsg));
    }

    getMessage(textMsg) {
        return JSON.stringify({
            text: textMsg,
        });
    }

    addToListaMessages(responseChat?: ResponseChatBot, isUser?: boolean) {
        this.formatAscii(responseChat);
        let messageChat: MessageChat;
        if (!isUser) {
            if (responseChat.attachment) {
                messageChat = {
                    bot: true,
                    user: false,
                    attachment: responseChat.attachment,
                };
            } else if (responseChat.quick_replies) {
                messageChat = {
                    bot: true,
                    user: false,
                    quick_replies: responseChat.quick_replies.map((button) => {
                        return { ...button, ...{ title: this.transformPipe.transform(button.title, CHATBOT.buttonTitleTruncate) } };
                    }),
                    text: responseChat.text.replace('"', ''),
                };
            } else if (responseChat.buttons) {
                messageChat = {
                    bot: true,
                    user: false,
                    quick_replies: responseChat.buttons.map((button) => {
                        return { ...button, ...{ title: this.transformPipe.transform(button.title, CHATBOT.buttonTitleTruncate) } };
                    }),
                    text: responseChat.text.replace('"', ''),
                };
            } else {
                messageChat = {
                    bot: true,
                    user: false,
                    text: responseChat.text.replace('"', ''),
                };
            }
        } else {
            messageChat = {
                bot: false,
                user: true,
                text: this.message_sent,
            };
        }
        this.listaMessages.push(messageChat);
        setTimeout(() => {
            this.content.scrollToBottom(200);
        }, 600);
    }

    quickReplies(question: string) {
        if (question !== '') {
            if (question.includes('http'))
                this.inAppBrowser.create(
                    question,
                    OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank',
                    'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes'
                );
            else this.handleMessage(question);
        }
    }

    doRefresh(event) {
        this.configSock();
        this.listaMessages = [];
        this.callServicesChat();
        setTimeout(() => {
            event.target.complete();
        }, 250);
    }

    startTimer() {
        setTimeout(() => {
            this.socketNotReady = false;
        }, CHATBOT.delayChat.delayPreChatTwo);
    }

    ngOnDestroy(): void {
        this.stompClient.disconnect({}, () => { });
    }

    formatAscii(chatMessage: ResponseChatBot) {
        if (chatMessage && chatMessage.text && chatMessage.text.match(ASCII_CHAR_REGEX)) {
            Object.entries(ASCII_CHARACTER)?.forEach((asciiArr) => {
                const incaseSensitiveReg = new RegExp(asciiArr[0], 'gi');
                if (chatMessage?.text?.match(incaseSensitiveReg)) {
                    chatMessage.text = chatMessage.text.replace(incaseSensitiveReg, asciiArr[1]);
                }
            });
        }
    }
}
