import { Location } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { AlertButton, ModalController, Platform } from '@ionic/angular';
import { pluck } from 'rxjs/operators';
import { ModalPdfViewerComponent } from 'src/app/components';
import { DocumentGp, DownloadFileRqGp, InfoUserProcedure, TaskGp, UserInfo, VariableGp } from 'src/app/models';
import { GlobalService, StorageService } from 'src/app/services';
import { GestorePraticheService } from 'src/app/services/gestore-pratiche/gestore-pratiche.service';
import { OPEN_WITH_EXTERNAL_BROWSER } from 'src/app/utility';
import { DURATIONTOAST, GP_STATUS, GPFORM, IMAGE, PAGES } from 'src/app/utility/global-constant';

@Component({
    selector: 'app-dettaglio-procedura',
    templateUrl: './dettaglio-procedura.page.html',
    styleUrls: ['./dettaglio-procedura.page.scss'],
})
export class DettaglioProceduraPage implements OnInit {
    @Input() procedure: InfoUserProcedure;
    titleHeader: string = PAGES.dettaglioProcedure.title;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    ios: boolean;
    android: boolean;
    procSel: string = GPFORM.procUteSel;
    listaTaskGp: Array<TaskGp>;
    listaVariablesGp: Array<VariableGp>;
    listaDocumentGp: Array<DocumentGp>;
    showSkeletonTasksGp: boolean = true;
    showSkeletonFileGp: boolean = true;
    showSkeletonVariablesGp: boolean = true;
    isOpenedInModal: boolean;
    hasParams: boolean;
    identifierRequester: string;
    idProcedure: number;
    userInfo: UserInfo;

    constructor(
        private platform: Platform,
        public location: Location,
        private globalService: GlobalService,
        private inAppBrowser: InAppBrowser,
        private storageService: StorageService,
        private gestorePraticheService: GestorePraticheService,
        private modalController: ModalController,
        private activatedRoute: ActivatedRoute
    ) {
        this.checkPlatform();
        this.setUtility();
        this.getUserInfo();
        this.getParams();
    }

    ngOnInit() {
        this.getUserInfo().then(() => {
            if (!this.hasParams) {
                this.isOpenedInModal = true;
                this.callServices();
            }
        });
    }

    getParams() {
        this.activatedRoute.params.pipe(pluck('params')).subscribe((result) => {
            this.procedure = this.globalService.decodeParams(result);
            if (this.procedure && this.procedure?.id) {
                this.hasParams = true;
                this.callServices();
            }
        });
    }

    checkPlatform() {
        this.ios = this.platform.is('ios');
        this.android = this.platform.is('android');
    }

    setUtility() {
        this.listaTaskGp = [];
        this.listaDocumentGp = [];
        this.listaVariablesGp = [];
    }

    getUserInfo() {
        return this.storageService.getUserInfo().then((userInfo) => (this.userInfo = userInfo));
    }

    callServices() {
        //this.getProcuderesVariables();
        this.getTaskByUser();
        this.getDocumentByProcedure();
    }

    getProcuderesVariables() {
        this.gestorePraticheService.getProcuderesVariables(this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonVariablesGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    if (result.item.length > 0) {
                        this.listaVariablesGp = result.item;
                    }
                }
            },
            () => {
                this.showSkeletonVariablesGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    getDocumentByProcedure() {
        this.gestorePraticheService.getDocumentByProcedure(this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonFileGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    if (result.item.results.length > 0) {
                        this.listaDocumentGp = result.item.results;
                    }
                }
            },
            () => {
                this.showSkeletonFileGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    getTaskByUser() {
        this.gestorePraticheService.getTaskByUser(this.procedure?.requester?.identifier, this.procedure?.id).subscribe(
            (result) => {
                this.showSkeletonTasksGp = false;
                if (result.status === GP_STATUS.status_ok) {
                    if ((result.item as unknown as TaskGp[]).length > 0) {
                        this.listaTaskGp = result.item as unknown as TaskGp[];
                    }
                }
            },
            () => {
                this.showSkeletonTasksGp = false;
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
            }
        );
    }

    downloadDocumentGp(file: DocumentGp) {
        this.gestorePraticheService.downloadDocumentGp(this.prepareRequestDownFile(file.id)).subscribe(
            (result) => {
                if (file.mimetype.includes('pdf')) {
                    this.openPdfViewrModal(result, file.name);
                } else {
                    this.confirmDownloadFile(file.name, result);
                }
            },
            () => this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default)
        );
    }

    downloadFile(file: DocumentGp) {
        /*   this.openUrl(this.globalService.b64toBlob(IMAGE_FITTIZIA.base_64, 'image/png')) */
        this.downloadDocumentGp(file);
    }

    openUrl(file: Blob) {
        const url = window.URL.createObjectURL(file);
        this.inAppBrowser.create(url, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
    }

    prepareRequestDownFile(idDocument: string): DownloadFileRqGp {
        return {
            documentId: idDocument,
            requesterIdentifier: this.procedure.requester.identifier,
        } as DownloadFileRqGp;
    }

    closeModalDetail() {
        this.modalController.dismiss();
    }

    async openPdfViewrModal(pdf: Blob, fileName: string) {
        let fileURL = URL.createObjectURL(pdf);
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: fileURL,
                base64File: pdf,
                fileName: fileName,
            },
        });
        return await modal.present();
    }

    confirmDownloadFile(fileName: string, fileBase64: Blob) {
        const buttons: AlertButton[] = [
            {
                text: 'No',
                handler: () => {
                    console.log('download annullato');
                },
            },
            {
                text: 'Si',
                cssClass: 'secondary',
                handler: () => {
                    this.globalService
                        .downloadFile(fileName, fileBase64)
                        .then((res) => {
                            // this.globalService.presentToast('Il file è stato salvato nella cartella Download del dispositivo', DURATIONTOAST.default);
                            this.globalService.openFile(res.nativeURL);
                        })
                        .catch((err) => {
                            this.globalService.presentToast('Impossibile scaricare il file', DURATIONTOAST.default);
                        });
                },
            },
        ];
        this.globalService.presentAlertConfirm('Conferma Download', 'Si desidera scaricare il file ?', buttons);
    }
}
