import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { IonContent, Platform } from '@ionic/angular';
import { FilterProcedureGp, InfoUserProcedure, ProcedureStatus, UserInfo } from 'src/app/models';
import { GlobalService, StorageService } from 'src/app/services';
import { GestorePraticheService } from 'src/app/services/gestore-pratiche/gestore-pratiche.service';
import { DURATIONTOAST, GP_STATUS, IMAGE, MESSAGES, PAGES, PAGINATION_USGP, PROCEDURE, SEARCH } from 'src/app/utility/global-constant';

@Component({
	selector: 'app-procedure',
	templateUrl: './procedure.page.html',
	styleUrls: ['./procedure.page.scss'],
})
export class ProcedurePage implements OnInit {
	@ViewChild(IonContent) content: IonContent;
	titleHeader: string = PAGES.procedure.title;
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	userInfo: UserInfo;
	proceduresFinished: boolean;
	page_number: number;
	listaProcedure: Array<InfoUserProcedure>;
	listaProcedurePlus: Array<InfoUserProcedure>;
	listaProcedureStatus: Array<ProcedureStatus> = [];
	notResult: boolean = false;
	textMessage: string = MESSAGES.notUserProcedure;
	statusProcSelected: string; //todo_select
	nameSearched: string;
	debounceSearch: number = SEARCH.debounce;
	ios: boolean;
	android: boolean;
	showSkeleton: boolean = true;
	sizeSkeleton: number = 4;
	fullscreen: boolean = true;
	placeholderSearchTitle: string = "Cerca";
	placeholderSearchState: string = "Cerca";

	constructor(
		private storageService: StorageService,
		private platform: Platform,
		private globalService: GlobalService,
		private gestorePraticheService: GestorePraticheService,
		private router: Router,
		private activatedRoute: ActivatedRoute
	) {
		this.checkPlatform();
		this.getUserInfo();
		this.setUtility();
	}

	ngOnInit() {
		// setTimeout(() => this.getFilterUserProcedures(), 300);
		this.getStatus();
	}

	ionViewDidEnter() {
		this.fullscreen = false;
	}

	ionViewWillLeave() {
		this.globalService.stopNgxLoader();
		this.fullscreen = true;
	}

	setUtility() {
		this.listaProcedure = [];
		this.listaProcedurePlus = [];
		this.setInitPagination();
		this.nameSearched = '';
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	setInitPagination() {
		this.page_number = PAGINATION_USGP.start_page;
		this.proceduresFinished = false;
	}

	setEndPagination() {
		this.proceduresFinished = true;
	}

	getUserInfo() {
		this.storageService.getUserInfo().then((userInfo) => {
			this.userInfo = userInfo;
		});
	}

	getFilterUserProcedures(plusElements?: boolean) {
		plusElements ? (this.page_number += 1) : null;
		this.gestorePraticheService.getUserProcedures(this.prepareRequestFUP()).subscribe(
			(result) => {
				this.showSkeleton = false;
				if (result.status === GP_STATUS.status_ok) {
					if (result.item.results.length > 0) {
						plusElements ? (this.listaProcedurePlus = result.item.results) : (this.listaProcedure = result.item.results);
					} else {
						plusElements ? this.setEndPagination() : this.setCardMessage(true);
					}
				} else {
					this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
					this.setEndPagination();
					this.setCardMessage(true);
				}
			},
			(error) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			}
		);
	}

	prepareRequestFUP(): FilterProcedureGp {
		let filterProcedureGp: FilterProcedureGp;
		filterProcedureGp = {
			identifier: this.userInfo.identifier,
			page: this.page_number,
			size: PAGINATION_USGP.items_per_page,
		};
		if (this.nameSearched && this.nameSearched !== '') {
			filterProcedureGp = this.nameSearched.length !== PROCEDURE.sizeProtocolNumber ? { ...filterProcedureGp, name: this.nameSearched } : { ...filterProcedureGp, protocolNumber: this.nameSearched }
		}
		filterProcedureGp = this.statusProcSelected ? { ...filterProcedureGp, status: this.statusProcSelected } : filterProcedureGp;
		return filterProcedureGp;
	}

	searchProcedureByNome(evento: Event) {
		this.setInitPagination();
		this.nameSearched = evento['detail']['value'] as string;
		this.forceContent();
		this.getFilterUserProcedures();
		return true;
	}

	searchProcedureByStatus(procedureStatus: ProcedureStatus) {
		this.statusProcSelected = procedureStatus.key;
		this.setInitPagination();
		this.forceContent();
		this.getFilterUserProcedures();
		return true;
	}

	setCardMessage(zeroResult: boolean) {
		if (zeroResult) {
			this.notResult = true;
			this.listaProcedure = [];
		}
	}

	doInfinite(event) {
		if (!this.proceduresFinished && !this.showSkeleton) {
			this.getFilterUserProcedures(true);
			setTimeout(() => {
				this.upgradeListaProcedures(this.listaProcedurePlus);
				event.target.complete();
			}, 1000);
		} else {
			event.target.complete();
		}
	}

	upgradeListaProcedures(listaProceduresPlus: Array<InfoUserProcedure>) {
		for (let procedure of listaProceduresPlus) {
			this.listaProcedure.push(procedure);
		}
		this.listaProcedurePlus = [];
	}

	doRefresh(event) {
		this.setInitPagination();
		this.getFilterUserProcedures();
		setTimeout(() => {
			event.target.complete();
		}, 250);
	}

	detailsProcedure(procedure: InfoUserProcedure) {
		let params = this.globalService.encodeParams(procedure);
		this.router.navigate([PAGES.dettaglioProcedure.link, params], { relativeTo: this.activatedRoute });
	}

	forceContent() {
		this.content.forceOverscroll = true;
	}

	getStatus() {
		this.gestorePraticheService.getStatuses().subscribe(
			(res) => {
				this.listaProcedureStatus = res.item;
				this.getFilterUserProcedures();
			},
			(err) => {
				this.showSkeleton = false;
				this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
				this.setEndPagination();
				this.setCardMessage(true);
			}
		);
	}

	onChangeStateSelect(data) {
		this.searchProcedureByStatus(data.value as ProcedureStatus);
	}

}
