import { Component, OnInit, ViewChild } from '@angular/core';
import { Coordinates, Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';
import { Platform } from '@ionic/angular';
import { Layer, LayerGroup, Map, MapOptions, Marker, TileLayer, circle, circleMarker, icon, latLng, tileLayer } from 'leaflet';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { CategoriaPoi, MarkerRequest, Poi } from 'src/app/models';
import { GlobalService, PoiService } from 'src/app/services';
import { UtilityService } from 'src/app/services/utility/utility.service';
import { CATPOI, DURATIONTOAST, IMAGE, MESSAGES, OPEN_STREET_MAPS, SEARCH } from 'src/app/utility/global-constant';

/* declare function initMap(geolocation, showErrorToastFunc);
declare function geocodeRestSearch(searchStr, multi_markers_amenity);
declare function clearLayers();
declare function getWatchPositionRef(): Subscriber<any>;
declare function getLocationIntervalRef(); */

@Component({
	selector: 'app-poi',
	templateUrl: './poi.page.html',
	styleUrls: ['./poi.page.scss'],
})
export class PoiPage implements OnInit {
	@ViewChild('map') map: Map;
	titleHeader: string = 'Mappa';
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	listaCategoriePoi: Array<CategoriaPoi>;
	listaCategoriePoiSave: Array<CategoriaPoi>;
	listaCategoriePoiFR: Array<CategoriaPoi>[];
	listaCategoriePoiFRR: Array<CategoriaPoi>[][];
	categoriaSelected: CategoriaPoi;
	ios: boolean;
	android: boolean;
	showMap: boolean = false;
	mapOptions: MapOptions;
	markers: Marker[] = [];
	markerMapLayerGroup = new LayerGroup();
	locationLayerGroup = new LayerGroup();
	baseMapLayerGroup = new LayerGroup();

	constructor(
		private platform: Platform,
		private poiService: PoiService,
		private utilityService: UtilityService,
		private globalService: GlobalService,
		private ngxService: NgxUiLoaderService,
		private geolocation: Geolocation) {
		this.checkPlatform();
		this.setUtility();
		this.getCategoriePoi();
	}

	ngOnInit() {
		this.initializeMapOptions();
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	ionViewWillEnter() {
		this.utilityService.updateSlidePoi$.next(true);
		this.showMap = true;
	}

	onMapReady(map: Map) {
		this.map = map;
		this.map.addLayer(this.createBaseLayer());
		this.map.addLayer(this.markerMapLayerGroup);
		this.map.addLayer(this.locationLayerGroup);
	}

	initializeMapOptions() {
		this.mapOptions = {
			center: latLng(OPEN_STREET_MAPS.center.lat, OPEN_STREET_MAPS.center.lng),
			zoom: 12,
			maxZoom: 18,
			zoomControl: false,
			preferCanvas: true,
			attributionControl: true,
		};
	}

	setUtility() {
		this.listaCategoriePoi = [];
		this.listaCategoriePoiSave = [];
		this.listaCategoriePoiFRR = [];
	}

	createBaseLayer(): LayerGroup {
		this.baseMapLayerGroup.clearLayers();
		const baseMapTileLayer = new TileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png');
		this.baseMapLayerGroup.addLayer(baseMapTileLayer);
		return this.baseMapLayerGroup;
	}

	addMarkers(markers: Poi[]) {
		markers.forEach((poi) => this.addSingleMarker(poi));
		this.markerMapLayerGroup.addTo(this.map)
	}

	addSingleMarker(poi: Poi) {
		const marker = new Marker([Number(poi.lat), Number(poi.lon)])
			.setIcon(
				icon({
					iconSize: [25, 32],
					iconAnchor: [13, 32],
					iconUrl: poi.icon ? poi.icon : 'assets/openmaps/markers_images/marker.png'
				}));
		marker.addTo(this.markerMapLayerGroup)
	}

	removeMarkers() {
		this.clearLayers();
	}

	setCenterMap(coords: Coordinates) {
		this.map.setView(latLng(coords.latitude, coords.longitude), 20);
	}

	setCategoriesPoi(categoriesPoi: CategoriaPoi[]) {
		this.listaCategoriePoi = categoriesPoi;
		this.listaCategoriePoiFRR = this.globalService.compositeForRowCatPoi(this.listaCategoriePoi, CATPOI.nForRow, CATPOI.maxRowFS);
		this.listaCategoriePoiSave = categoriesPoi;
	}


	filterFromCat(categoria: CategoriaPoi) {
		this.categoriaSelected = categoria;
		if (categoria.b_selected) { 
			this.removeMarkers();
			this.getMarkers(this.prepareMarkerRequest(categoria.nCatPoi, OPEN_STREET_MAPS.enableBoxView ? this.map.getBounds().toBBoxString() : null));
	 	} else {
			this.removeMarkers();
		} 
	}

	locate() {
		this.startAndStopLoader(true);
		this.geolocation.getCurrentPosition().then(
			(result) => {
				this.onLocationSuccess(result.coords);
			},
			() => {
				this.globalService.presentToast(MESSAGES.locationUnavailable, DURATIONTOAST.default);
				this.startAndStopLoader();
			},
		);
	}

	onLocationSuccess(coords: Coordinates) {
		this.locationLayerGroup.clearLayers();
		const { accuracy, latitude, longitude } = coords;
		const latlng = [latitude, longitude];
		this.setCenterMap(coords);
		const locationCircle = circle(latlng as any, accuracy, {
			fillOpacity: 0.2,
			color: '#4285f4'
		});
		this.locationLayerGroup.getLayers().length == 0 && this.locationLayerGroup.addLayer(locationCircle)
		this.startAndStopLoader();
	}

	clearLayers() {
		this.markerMapLayerGroup.clearLayers();
	}

	startAndStopLoader(start?: boolean) {
		start ? this.ngxService.start() : this.ngxService.stop();
	}

	ionViewWillLeave() {
		this.showMap = false;
	}

	getMarkers(markerRequest: MarkerRequest) {
		this.poiService.getMarkers(markerRequest).subscribe(
			{
				next: (result) => {
					this.addMarkers(result)
				},
				error: (error) => {
					this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
				},
			}
		);
	}

	getCategoriePoi() {
		this.poiService.getLocalCategoriePoi().subscribe({
			next: (result) => {
				this.setCategoriesPoi(result);
				this.getRemoteCategoriePoi();
			},
			error: (error) => {
				this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
			},
		});
	}

	getRemoteCategoriePoi() {
		this.poiService.getCategoriePoi().subscribe({
			next: (result) => {
				this.setCategoriesPoi(result.data);
			},
			error: (error) => {
				//this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
			},
		});
	}

	prepareMarkerRequest(category: string, boxView?: string): MarkerRequest {
		let markerRequest: MarkerRequest = {
			category: category
		};
		return boxView ? {
			...markerRequest, ...{ boxView: boxView }
		} : markerRequest
	}

}
