import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { AuthService, IAuthAction } from 'ionic-appauth';
import { camelCase } from 'lodash';
import { from, Observable, Subscription } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';
import { ClaimsUserIdp, TokenResponseIdp, UserInfo } from 'src/app/models';
import { CONSOLE, PAGES, REFRESH_TOKEN_AUTHCONFIG, SILENT_AUTHCONFIG } from 'src/app/utility';

import { StorageService, UtilityService } from '../';
import { AuthenticationService } from '../authentication/authentication.service';

@Injectable({
	providedIn: 'root',
})
export class IdpInformationService {
	events$ = this.auth.events$;
	sub: Subscription;
	user$ = this.auth.user$;

	constructor(
		private auth: AuthService,
		private utilityService: UtilityService,
		private storageService: StorageService,
		private router: Router,
		private http: HttpClient,
		private authenticationService: AuthenticationService
	) {
		this.subscribeInfoUserAndSave();
		this.subscribeTokenAndSave();
	}

	public getUserInfo(): Promise<void> {
		return this.auth.loadUserInfo();
	}

	public getRefreshedTokenObservable() {
		return from(this.refreshToken()) as Observable<TokenResponseIdp>;
	}

	public generateSilentToken() {
		const { body: authBody, headers: authHeaders, endpoint } = SILENT_AUTHCONFIG;
		let body = new HttpParams();
		for (const [key, value] of Object.entries(authBody)) {
			body = body.append(key, value);
		}
		const headers = new HttpHeaders(authHeaders);
		return this.http.post<any>(endpoint, body.toString(), { headers }).pipe(this.handleTokenResponse());
	}

	public refreshToken() {
		return from(this.storageService.getTokenIdp()).pipe(
			switchMap(({ refreshToken }) => {
				const { refreshBody, headers: authHeaders, endpoint } = REFRESH_TOKEN_AUTHCONFIG;
				let body = new HttpParams();
				for (const [key, value] of Object.entries(refreshBody)) {
					body = body.append(key, value ?? refreshToken);
				}
				const headers = new HttpHeaders(authHeaders);
				return this.http.post<any>(endpoint, body.toString(), { headers }).pipe(this.handleTokenResponse());
			}),
			catchError((err) => {
				this.router.navigate([PAGES.landing_page], { replaceUrl: true });
				this.authenticationService.logout();
				return err;
			})
		);
	}

	private handleTokenResponse = () =>
		map(
			(response: any) => {
				delete response.token_type;
				let sanitizedResponse: object;
				for (const [key, value] of Object.entries(response)) {
					sanitizedResponse = {
						[camelCase(key)]: value,
						...sanitizedResponse,
					};
				}
				this.storageService.storeTokenIdp(sanitizedResponse as TokenResponseIdp);
				return sanitizedResponse;
			},
			(_err) => {
				CONSOLE.log && console.log('User authentication failed!');
			}
		);

	subscribeInfoUserAndSave() {
		this.user$.subscribe((claimsUserIdp: ClaimsUserIdp) => {
			CONSOLE.log && console.log('[IdpInformationService - subscribeInfoUserAndSave - claimsUserIdp]: ', claimsUserIdp);
			if (claimsUserIdp) {
				this.storageService.storeUserInfo(this.prepareInfoUser(claimsUserIdp)).then((result) => {
					CONSOLE.log && console.log('[IdpInformationService - subscribeInfoUserAndSave - resultStore]: ', result);
					if (result) {
						this.utilityService.sendTokenFirebase();
						/*          this.utilityService.getDettaglioUtenza(); */
					}
				});
			}
		});
	}

	prepareInfoUser(claimsUserIdp: ClaimsUserIdp): UserInfo {
		return {
			identifier: claimsUserIdp.sub,
			firstName: claimsUserIdp.name,
			lastName: claimsUserIdp.family_name,
			email: claimsUserIdp.email,
			family_name: claimsUserIdp.family_name,
			sub: claimsUserIdp.sub,
			gender: claimsUserIdp?.gender ? claimsUserIdp.gender : null,
			birthdate: claimsUserIdp?.birthdate ? claimsUserIdp.birthdate : null,
		};
	}

	subscribeTokenAndSave() {
		this.events$.subscribe((actionToken: IAuthAction) => {
			if (actionToken?.tokenResponse && actionToken?.tokenResponse?.accessToken) {
				CONSOLE.log && console.log('[IdpInformationService - subscribeTokenAndSave - token]', actionToken?.tokenResponse);
				this.storageService.storeTokenIdp(actionToken?.tokenResponse as TokenResponseIdp);
			}
		});
	}

	saveUserInformation() {
		this.getUserInfo().then();
	}

	ngOnDestroy() {
		this.sub.unsubscribe();
	}
}
