package it.itsvil.notificatore.backend.service;

import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Scanner;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.auth.oauth2.GoogleCredentials;
import com.google.gson.Gson;
import com.google.gson.JsonObject;

import it.itsvil.notificatore.backend.dao.NotificatoreDao;
import it.itsvil.notificatore.backend.dto.BroadcastPushNotificationRequestDTO;
import it.itsvil.notificatore.backend.dto.PunctualPushNotificationDTO;
import it.itsvil.notificatore.backend.dto.PushNotificationDTO;
import it.itsvil.notificatore.backend.dto.TokenDTO;
import it.itsvil.notificatore.backend.exceptions.InitPushNotificationException;
import it.itsvil.notificatore.backend.utils.NotificatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "NotificatorePushNotificationServiceImpl")
public class NotificatorePushNotificationServiceImpl implements NotificatorePushNotificationService{

	private static final String MESSAGING_SCOPE = "https://www.googleapis.com/auth/firebase.messaging";

	@Autowired
	NotificatoreDao dao;

	@Override
	public void loadConfiguration(String n_id_app, PushNotificationDTO request) throws InitPushNotificationException {
		log.info("RECUPERO DELLE CONFIGURAZIONI OBBLIGATORIE");

		String firebaseServerUrl = (dao.getKeyforPushNotification(n_id_app,NotificatoreEnum.FIREBASE_SERVER_URL.getCodice())).getT_value();
		String firebaseServerKeyAndroid = (dao.getKeyforPushNotification(n_id_app,NotificatoreEnum.FIREBASE_SERVER_KEY_ANDROID.getCodice())).getT_value();
		String firebaseServerKeyIos = (dao.getKeyforPushNotification(n_id_app,NotificatoreEnum.FIREBASE_SERVER_KEY_IOS.getCodice())).getT_value();
		if(StringUtils.isBlank(firebaseServerUrl)) {
			throw new InitPushNotificationException("Parametro "+NotificatoreEnum.FIREBASE_SERVER_URL.getCodice()+ " non presente");
		}
		if(StringUtils.isBlank(firebaseServerKeyAndroid)) {
			throw new InitPushNotificationException("Parametro "+NotificatoreEnum.FIREBASE_SERVER_KEY_ANDROID.getCodice()+ " non presente");		
		}
		if(StringUtils.isBlank(firebaseServerKeyIos)) {
			throw new InitPushNotificationException("Parametro "+NotificatoreEnum.FIREBASE_SERVER_KEY_IOS.getCodice()+ " non presente");
		}

		log.info("CONFIGURAZIONI OBBLIGATORIE PRESENTI");
	}

	@Override
	public boolean sendBroadcastPushNotification(String n_id_app, BroadcastPushNotificationRequestDTO request) throws Exception {
		log.info("START");
		
		boolean esito = false; 
		List<TokenDTO> tipo = request.getListOfToken();
		
		for(TokenDTO token : tipo) {
			log.info("INVIO NOTIFICA PER: {}", new Gson().toJson(token));
			esito = sendSimplePushNotification(token.getSistema_operativo(), n_id_app, token.getToken(), request.getPushTitle(), request.getPushMessage(), request.getParametriAggiuntivi());
		}
		log.info("ESITO FINALE: {}", esito);
		
		log.info("END");
		return esito;
	}
	
	@Override
	public boolean sendPushNotification(String applicationId, PushNotificationDTO push) throws Exception{
		log.info("START");
		
		boolean esito = false; 
		List<PunctualPushNotificationDTO> notificationList = push.getNotifications();
		
		for(PunctualPushNotificationDTO p : notificationList) {
			TokenDTO token = p.getToken();
			log.info("INVIO NOTIFICA PER: {}", new Gson().toJson(token));
			esito = sendSimplePushNotification(token.getSistema_operativo(), applicationId, token.getToken(), p.getPushTitle(), p.getPushMessage(), p.getParametriAggiuntivi());
		}
		log.info("ESITO FINALE: {}", esito);
		
		log.info("END");
		return esito;
	}
	
	@Override
	public boolean checkSendPushNotificationInput(PushNotificationDTO push) {
		boolean esito = true;
		if(null != push && null != push.getNotifications() && push.getNotifications().size() > 0){
			List<PunctualPushNotificationDTO> punctualList = push.getNotifications();
			for (PunctualPushNotificationDTO punctualPushNotificationDTO : punctualList) {
				TokenDTO token = punctualPushNotificationDTO.getToken();
				if(null == token || StringUtils.isBlank(token.getSistema_operativo()) || StringUtils.isBlank(token.getToken()) ||
						StringUtils.isBlank(punctualPushNotificationDTO.getPushMessage()) || StringUtils.isBlank(punctualPushNotificationDTO.getPushTitle())){
					return false;
				}
			}
		}else{
			return false;
		}
		return esito;
	}
	
	/**
	 * Metodo che effettua l'invio della notifica
	 * @param sistema_operativo indica il sistema operativo del dispositivo a cui bisogna inviare la notifica
	 * @param n_id_app identificativo univoco dell'app
	 * @param firebaseTokenDevicetoken che identifica il dispositivo a cui inviare la notifica
	 * @param pushTitle titolo della notifica
	 * @param pushMessage corpo della notifica
	 * @param parametriAggiuntivi info aggiuntive della notifica
	 * @return true se la notifica push è stata inviata correttamente, false altrimenti
	 * @throws Exception
	 */
	private boolean sendSimplePushNotification(String sistema_operativo, String n_id_app, String firebaseTokenDevice, String pushTitle, 
			String pushMessage, HashMap<String,String> parametriAggiuntivi) throws Exception{
		
		log.info("START");
		
		String serviceAccount = (dao.getKeyforPushNotification(n_id_app, NotificatoreEnum.SERVICE_ACCOUNT.getCodice())).getT_value();
		log.info("serviceAccount: {}", serviceAccount);
		if(StringUtils.isBlank(serviceAccount)) {
			log.info("SERVICEACCOUNT NON RECUPERATO.");
		}
		
		JsonObject notificationToSend = getNotification(sistema_operativo, firebaseTokenDevice, pushTitle, pushMessage, parametriAggiuntivi);
		HttpURLConnection connection = getConnection(n_id_app, serviceAccount);
		
		connection.setDoOutput(true);
	    DataOutputStream outputStream = new DataOutputStream(connection.getOutputStream());
	    outputStream.writeBytes(notificationToSend.toString());
	    
	    outputStream.flush();
	    outputStream.close();
	    
	    int responseCode = connection.getResponseCode();
	    String responseMessage = connection.getResponseMessage();
	    
	    log.info("responseCode: {}", responseCode);
	    log.info("responseMessage: {}", responseMessage);
	    
	    if (responseCode == 200) {
	    	String response = inputstreamToString(connection.getInputStream());
	      	log.info("RESPONSE: {}", response);
	      	return true;
	    } 
	    else {
	      	String response = inputstreamToString(connection.getErrorStream());
	      	log.info("RESPONSE: {}", response);
	      	return false;
	    }
	}
	
	/**
	 * Metodo utile alla creazione di una connessione di tipo HttpURLConnection
	 * @param n_id_app identificativo univoco dell'applicazione 
	 * @return una connessione
	 * @throws Exception
	 */
	private HttpURLConnection getConnection(String n_id_app, String serviceAccount) throws Exception{
		log.info("START");
		
		//Recupero URL di firebase
		String firebaseURL = (dao.getKeyforPushNotification(n_id_app, NotificatoreEnum.FIREBASE_SERVER_URL_HTTP_V1.getCodice())).getT_value();
		log.info("FIREBASEURL: {}", firebaseURL);
		if(StringUtils.isBlank(firebaseURL)){
			log.info("FIREBASE_SERVER_URL_HTTP_V1 non recuperato");
			throw new Exception("FIREBASE_SERVER_URL_HTTP_V1 non recuperato");
		}

		//Recupero del token
		String token = getAccessToken(serviceAccount);
		log.info("token: {}", token);
		if(StringUtils.isBlank(token)){
			log.info("TOKEN NON GENERATO");
			throw new Exception("Token non generato");
		}
		
		URL connectionURL = new URL(firebaseURL); 
		HttpURLConnection httpURLConnection = (HttpURLConnection) connectionURL.openConnection();
		httpURLConnection.setRequestMethod("POST");
	    httpURLConnection.setRequestProperty("Authorization", "Bearer " + token);
	    httpURLConnection.setRequestProperty("Content-Type", "application/json; UTF-8");
		
	    log.info("END");
	    return httpURLConnection;
	}
	
	/**
	 * Metodo utile alla costruzione di una semplice notifica da inviare
	 * @param sistema_operativo indica il sistema operativo del dispositivo verso il quale bisogna inviare la notifica push
	 * @param firebaseTokenDevice token del dispositivo a cui bisogna inviare la notifica push
	 * @param pushTitle titolo della notifica push
	 * @param pushMessage messaggio della notifica push
	 * @param parametriAggiuntivi mappa contenente delle possibili informazioni aggiuntive da inserire nel corpo della notifica
	 * @return un oggetto Json contenente il messaggio da inviare
	 */
	private JsonObject getNotification(String sistema_operativo, String firebaseTokenDevice, String pushTitle, String pushMessage, HashMap<String,String> parametriAggiuntivi) {
		log.info("START");
		
		JsonObject jNotification = new JsonObject(); //Notifica generale, valida per entrambi i SO
		JsonObject jMessage = new JsonObject(); //Messaggio contenente la notifica, i parametri aggiuntivi, il destinatario ed eventuali info in base al SO 
		JsonObject jFinalMessage = new JsonObject(); //Messaggio completo
	    
		JsonObject jMessageApns = new JsonObject(); 
		JsonObject jPayloadApns = new JsonObject();
		JsonObject jAps = new JsonObject();
		
		//Settaggio parametri della notifica principale
	    jNotification.addProperty("title", pushTitle);
	    jNotification.addProperty("body", pushMessage);
	    
	    jMessage.add("notification", jNotification);
	    
	    //Verifica ed inserimento di eventuali info aggiuntive
	    if(null != parametriAggiuntivi && !parametriAggiuntivi.isEmpty()) {
	    	log.info("PARAMETRI AGGIUNTIVI PRESENTI");
	    	
	    	JsonObject data = new JsonObject(); //JSon per le info aggiutive, fratello di notification
	    	Iterator<Entry<String, String>> iterator = parametriAggiuntivi.entrySet().iterator();
	    	
	    	while (iterator.hasNext()) {
				Map.Entry<String, String> pairs = (Map.Entry<String, String>) iterator.next();
				log.info("INFO AGGIUNTIVA -> Chiave: {} Valore: {}", pairs.getKey(), pairs.getValue());
				data.addProperty(pairs.getKey(), pairs.getValue());
			}
	    	
	    	jMessage.add("data", data);
	    }
	    
	    //Nel caso in cui la notifica è diretta ad un dispositivo Android allora viene aggiunto un nuovo oggetto fratello a notification
    	if(sistema_operativo.equalsIgnoreCase(NotificatoreEnum.ANDROID.getCodice())) {
    		JsonObject jMessageAndroid = new JsonObject(); //Fratello di notification da aggiungere
    		JsonObject jNotificationAndroid = new JsonObject(); //Info aggiuntive alla notificaper Android
    		
    		jNotificationAndroid.addProperty("click_action", NotificatoreEnum.FCM_PLUGIN_ACTIVITY);
    		jMessageAndroid.add("notification", jNotificationAndroid);
    		jMessage.add("android", jMessageAndroid);
    	}
    	
    	//Aggiunta di APNS
    	jAps.addProperty("category", NotificatoreEnum.NEW_MESSAGE_CATEGORY.getCodice());
    	jPayloadApns.add("aps", jAps);
    	jMessageApns.add("payload", jPayloadApns);
    	jMessage.add("apns", jMessageApns);
	    
	    jMessage.addProperty("token", firebaseTokenDevice);
	    
	    jFinalMessage.add("message", jMessage);
	    log.info("NOTIFICA COMPLETA: {}", new Gson().toJson(jFinalMessage));
	    
	    log.info("END");
	    return jFinalMessage;
	}
	
	/**
	 * Metodo utile a convertire un flusso di tipo InputStream in String
	 * @param inputStream flusso da convertire
	 * @return una stringa contenente il contenuto del flusso
	 * @throws IOException
	 */
	private static String inputstreamToString(InputStream inputStream) throws IOException {
		StringBuilder stringBuilder = new StringBuilder();
		Scanner scanner = new Scanner(inputStream);
		while (scanner.hasNext()) {
			stringBuilder.append(scanner.nextLine());
		}
		scanner.close();
		
		return stringBuilder.toString();
	}
	
	/**
	 * Metodo utile al recupero di un token di accesso valido
	 * @param serviceAccount contenuto del file JSon recuperato dalla console firebase del progetto (Impostazioni>Account di servizio>Genera nuova chiave privata 
	 * @return token
	 * @throws IOException
	 */
	private static String getAccessToken(String serviceAccount) throws IOException {
		GoogleCredentials googleCredential = GoogleCredentials
				.fromStream(new ByteArrayInputStream(serviceAccount.getBytes())) //Prende un inputStream
				.createScoped(Arrays.asList(MESSAGING_SCOPE));
		  
		googleCredential.refreshIfExpired();
		  
		return googleCredential.getAccessToken().getTokenValue();
	}

}
